@@ -61,9 +61,9 @@ except ImportError:
         pass
 from datetime import datetime
 from functools import partial
+import inspect
 import io
 import json
-import multiprocessing
 from multiprocessing import Process, Value, Array
 import os
 import subprocess
@@ -73,13 +73,14 @@ from threading import Thread
 import time
 import traceback
 from typing import Callable, Coroutine, Awaitable, Optional, Union, Dict, List, Tuple, Sequence, \
-    Set, Any, cast, Type, TypeVar, Generic
+    Set, Any, cast, Type, TypeVar
 
 from DHParser.configuration import access_thread_locals, get_config_value
-from DHParser.syntaxtree import DHParser_JSONEncoder
+from DHParser.nodetree import DHParser_JSONEncoder
 from DHParser.log import create_log, append_log, is_logging, log_dir
 from DHParser.toolkit import re, re_find, JSON_Type, JSON_Dict, JSONstr, JSONnull, \
-    json_encode_string, json_rpc, json_dumps, identify_python
+    json_encode_string, json_rpc, json_dumps, identify_python, normalize_docstring, md5, \
+    is_html_name
 from DHParser.versionnumber import __version__
 
 
@@ -101,8 +102,12 @@ __all__ = ('RPC_Table',
            'IDENTIFY_REQUEST',
            'IDENTIFY_REQUEST_BYTES',
            'LOGGING_REQUEST',
+           'INFO_REQUEST',
+           'SERVE_REQUEST',
            'SERVER_REPLY_TIMEOUT',
            'ALL_RPCs',
+           'rpc_entry_info',
+           'rpc_table_info',
            'pp_json',
            'pp_json_str',
            'asyncio_run',
@@ -152,7 +157,7 @@ Server: DHParser
 Accept-Ranges: none
 Content-Length: {length}
 Connection: close
-Content-Type: text/html; charset=utf-8
+Content-Type: {mime}; charset=utf-8
 X-Pad: avoid browser bug
 
 '''
@@ -160,20 +165,21 @@ X-Pad: avoid browser bug
 JSONRPC_HEADER_BYTES = b'''Content-Length: %i\r\n\r\n'''
 JSONRPC_HEADER = '''Content-Length: %i\r\n\r\n'''
 
-ONELINER_HTML = '''<!DOCTYPE html>
+HTML_TEMPLATE = '''<!DOCTYPE html>
 <html lang="en" xml:lang="en">
 <head>
-  <meta charset="UTF-8" />
+  <meta charset="utf-8" />
 </head>
 <body>
-<h1>{line}</h1>
+<h1>{heading}</h1>
+{content}
 </body>
 </html>
-
 '''
 
-UNKNOWN_FUNC_HTML = ONELINER_HTML.format(
-    line="DHParser Error: Function {func} unknown or not registered!")
+UNKNOWN_FUNC_HTML = HTML_TEMPLATE.format(
+    heading="DHParser Error: Function &ldquo;{func}&rdquo; unknown or not registered!",
+    content='')
 
 USE_DEFAULT_HOST = ''
 USE_DEFAULT_PORT = -1
@@ -182,6 +188,8 @@ STOP_SERVER_REQUEST = "__STOP_SERVER__"
 STOP_SERVER_REQUEST_BYTES = b"__STOP_SERVER__"
 IDENTIFY_REQUEST = "identify()"
 IDENTIFY_REQUEST_BYTES = b"identify()"
+INFO_REQUEST = "info()"
+SERVE_REQUEST = "serve()"
 LOGGING_REQUEST = "logging('')"
 
 SERVER_REPLY_TIMEOUT = 3  # seconds
@@ -278,7 +286,7 @@ def pp_json(obj: JSON_Type, *, cls=json.JSONEncoder) -> str:
             return [obj.serialized_json]
         elif isinstance(obj, JSONnull) or obj is JSONnull:
             return ['null']
-        return serialize(custom_encoder.default(obj))
+        return serialize(custom_encoder.default(obj), indent)
 
     return ''.join(serialize(obj, ''))
 
@@ -325,7 +333,7 @@ async def asyncio_connect(
     delay = retry_timeout / 1.5**12 if retry_timeout > 0.0 else retry_timeout - 0.001
     connected = False
     reader, writer = None, None
-    save_error = ConnectionError  # type: Union[Type[ConnectionError], ConnectionRefusedError]
+    save_error = ConnectionError  # type: Union[Type[ConnectionError], ConnectionRefusedError, OSError]
     OSError_countdown = 10
     while not connected and delay < retry_timeout:
         try:
@@ -361,21 +369,67 @@ def GMT_timestamp() -> str:
 ALL_RPCs = set('*')  # Magic value denoting all remote procedures
 
 
+def _func_info(name: str, func: Callable, html: bool) -> List[str]:
+    """Internal function to extract signature and docstring from
+    a function."""
+    info = []
+    if not name:  name = func.__name__
+    info.append(name + str(inspect.signature(func)))
+    if html:  info[-1] = '<b>' + info[-1] + '</b>'
+    docstr = normalize_docstring(getattr(func, '__doc__', '') or '')
+    if html:  docstr = '<br/>\n'.join(docstr.split('\n'))
+    if docstr:
+        info.extend(['', docstr, '', ''])
+    else:
+        info.append('')
+    return info
+
+
+def _merge_info(info: List[str], html: bool) -> str:
+    """Interrnal function to merge function infos into
+    a string, omitting trailing empty lines."""
+    while info and not info[-1]:  info.pop()
+    if html:
+        return '\n'.join(['<samp>', '<br/>\n'.join(info), '</samp>'])
+    return '\n'.join(info)
+
+
+def rpc_entry_info(name: str, rpc_table: RPC_Table, html: bool=False) -> str:
+    """Returns the name, signature and doc-string of a function in the
+    rpc-table as string or HTML-snippet."""
+    info = _func_info(name, rpc_table[name], html)
+    return _merge_info(info, html)
+
+
+def rpc_table_info(rpc_table: RPC_Table, html: bool=False) -> str:
+    """Returns the names, function signatures and doc-string of all
+    functions in the `rpc_table` as a (more or less) well-formatted
+    string or as HTML-snippet."""
+    info = []
+    for name, func in rpc_table.items():
+        info.extend(_func_info(name, func, html))
+    while info and not info[-1]:  info.pop()
+    return _merge_info(info, html)
+
+
 def default_fallback(*args, **kwargs) -> str:
     return 'No default RPC-function defined!'
 
 
-def http_response(html: str) -> bytes:
-    """Embeds an html-string in a http header and returns the http-package
+def http_response(html: Union[str, bytes], mime_type: str = 'text/html') -> bytes:
+    """Embeds an HTML-string in a http header and returns the http-package
     as byte-string.
     """
     gmt = GMT_timestamp()
     if isinstance(html, str):
         encoded_html = html.encode()
+    elif isinstance(html, bytes):
+        encoded_html = html
     else:
-        encoded_html = "Illegal type %s for response %s. Only str allowed!" \
-                       % (str(type(html)), str(html))
-    response = HTTP_RESPONSE_HEADER.format(date=gmt, length=len(encoded_html))
+        encoded_html = ("Illegal type %s for response %s. Only str allowed!" \
+                        % (str(type(html)), str(html))).encode()
+        mime_type = 'text/html'
+    response = HTTP_RESPONSE_HEADER.format(date=gmt, length=len(encoded_html), mime=mime_type)
     return response.encode() + encoded_html
 
 
@@ -461,7 +515,7 @@ class ExecutionEnvironment:
         self._process_executor = None                  # type: Optional[ProcessPoolExecutor]
         self._thread_executor = None                   # type: Optional[ThreadPoolExecutor]
         self._submit_pool = None                       # type: Optional[ProcessPoolExecutor]
-        self.submit_pool_lock = threading.Lock()       # type: multiprocessing.Lock
+        self.submit_pool_lock = threading.Lock()       # type: threading.Lock
         self.loop = event_loop                         # type: asyncio.AbstractEventLoop
         self.log_file = ''                             # type: str
         self._closed = False                           # type: bool
@@ -496,7 +550,7 @@ class ExecutionEnvironment:
     async def execute(self, executor: Optional[Executor],
                       method: Callable,
                       params: Union[dict, tuple, list])\
-            -> Tuple[JSON_Type, Optional[RPC_Error_Type]]:
+            -> Tuple[Union[JSON_Type, BytesType], Optional[RPC_Error_Type]]:
         """Executes a method with the given parameters in a given executor
         (``ThreadPoolExcecutor`` or ``ProcessPoolExecutor``). ``execute()`` waits
         for the completion and returns the JSON result and an RPC error tuple (see
@@ -509,7 +563,7 @@ class ExecutionEnvironment:
                           "Server Error: Execution environment has already been shut down! "
                           "Cannot process method {} with parameters {} any more."
                           .format(method, params))
-        result = None      # type: Optional[JSON_Type]
+        result = None      # type: Optional[Union[JSON_Type, BytesType]]
         rpc_error = None   # type: Optional[RPC_Error_Type]
         if params is None:
             params = tuple()
@@ -542,7 +596,7 @@ class ExecutionEnvironment:
             try:
                 # restart process pool and try again once
                 self.process_executor.shutdown(wait=True)
-                self.process_executor = ProcessPoolExecutor()
+                self._process_executor = ProcessPoolExecutor()
                 result = await self.loop.run_in_executor(executor, executable)
             except BrokenProcessPool as e:
                 rpc_error = -32050, str(e)
@@ -553,7 +607,7 @@ class ExecutionEnvironment:
             try:
                 # restart thread pool and try again once
                 self.thread_executor.shutdown(wait=True)
-                self.thread_executor = ThreadPoolExecutor()
+                self._thread_executor = ThreadPoolExecutor()
                 result = await self.loop.run_in_executor(executor, executable)
             except BrokenThreadPool as e:
                 rpc_error = -32060, str(e)
@@ -564,11 +618,11 @@ class ExecutionEnvironment:
         return result, rpc_error
 
     def submit_as_process(self, func, *args) -> Future:
-        """Submits a running long running function to the secondary process-pool.
+        """Submits long running function to the secondary process-pool.
         Other than `execute()` this works synchronously and thread-safe.
         """
         if self.submit_pool is None:
-            self.submit_pool = ProcessPoolExecutor()
+            self._submit_pool = ProcessPoolExecutor()
         with self.submit_pool_lock:
             future = self.submit_pool.submit(func, *args)
         return future
@@ -603,7 +657,7 @@ class StreamReaderProxy:
         except AttributeError:
             self.buffered_io = io_reader
         self.loop = None    # type: Optional[asyncio.AbstractEventLoop]
-        self.exec = None    # type: Optional[concurrent.futures.Executor]
+        self.exec = None    # type: Optional[Executor]
         self.max_data_size = get_config_value('max_rpc_size')
         self._eof = False   # type: bool
 
@@ -691,7 +745,6 @@ async def read_full_block(reader: StreamReaderType) -> Tuple[int, bytes, bytes]:
     return header_size, data, backlog
 
 
-
 class StreamWriterProxy:
     """StreamWriterProxy simulates an asyncio.StreamWriter that sends
     and receives data through an io.IOBase-Stream.
@@ -706,7 +759,7 @@ class StreamWriterProxy:
             self.buffered_io = io_writer
         self.buffer = []
         self.loop = None    # type: Optional[asyncio.AbstractEventLoop]
-        self.exec = None    # type: Optional[concurrent.futures.Executor]
+        self.exec = None    # type: Optional[Executor]
 
     def write(self, data: bytes):
         assert isinstance(data, bytes)
@@ -924,7 +977,7 @@ class Server:
     Language-Server-Protocol-Server. Class Server should be
     considered final, i.e. do not derive from this class to add
     LSP-functionality, rather implement the lsp_functionality in
-    a dedicated class (or set of cuntions) and pass the
+    a dedicated class (or set of functions) and pass the
     LSP-functionality via the rpc_functions-parameter to the
     constructor of this class.
 
@@ -955,6 +1008,9 @@ class Server:
     :var serving_task: The task in which the asyncio.Server is run.
     :var stop_response:  The response string that is written to the stream
         as answer to a stop request.
+    :var service_calls:  A set of names of functions that can be called
+        as "service calls" from a second connection, even if another
+        connection is still open.
     :var echo_log: Read from the global configuration. If True, any log
         message will also be echoed on the console.
     :var log_file: The file-name of the server-log.
@@ -968,7 +1024,7 @@ class Server:
     :var connection: An instance of the connection class representing the
         data of the current connection or None, if there is no connection at
         the moment. There can be only one connection to the server at a time!
-    :var kill_switch: If True the, the server will be shut down.
+    :var kill_switch: If True, the server will be shut down.
     :var loop: The asyncio event loop within which the asyncio stream server
         is run.
     """
@@ -1000,8 +1056,8 @@ class Server:
         assert STOP_SERVER_REQUEST not in self.rpc_table
 
         # see: https://docs.python.org/3/library/asyncio-eventloop.html#executing-code-in-thread-or-process-pools
-        self.cpu_bound = frozenset(self.rpc_table.keys()) if cpu_bound == ALL_RPCs else cpu_bound
-        self.blocking = frozenset(self.rpc_table.keys()) if blocking == ALL_RPCs else blocking
+        self.cpu_bound = set(self.rpc_table.keys()) if cpu_bound == ALL_RPCs else cpu_bound
+        self.blocking = set(self.rpc_table.keys()) if blocking == ALL_RPCs else blocking
         self.blocking -= self.cpu_bound  # cpu_bound property takes precedence
 
         assert not (self.cpu_bound - self.rpc_table.keys())
@@ -1019,17 +1075,20 @@ class Server:
 
         # if the server is run in a separate process, the following variables
         # should only be accessed from the server process
-        self.server = None        # type: Optional[asyncio.AbstractServer]
-        self.serving_task = None  # type: Optional[asyncio.Task]
-        self.stop_response = ''   # type: str
+        self.server = None          # type: Optional[asyncio.AbstractServer]
+        self.serving_task = None    # type: Optional[asyncio.Task]
+        self.stop_response = ''     # type: str
+        self.service_calls = set()  # type: Set[str]
 
         self.register_service_rpc(IDENTIFY_REQUEST, self.rpc_identify_server)
         self.register_service_rpc(LOGGING_REQUEST, self.rpc_logging)
+        self.register_service_rpc(INFO_REQUEST, self.rpc_info)
+        self.register_service_rpc(SERVE_REQUEST, self.rpc_serve_page)
 
-        self.exec = None          # type: Optional[ExecutionEnvironment]
-        self.connection = None    # type: Optional[Connection]
-        self.kill_switch = False  # type: bool
-        self.loop = None          # type: Optional[asyncio.AbstractEventLoop]
+        self.exec = None            # type: Optional[ExecutionEnvironment]
+        self.connection = None      # type: Optional[Connection]
+        self.kill_switch = False    # type: bool
+        self.loop = None            # type: Optional[asyncio.AbstractEventLoop]
 
         self.known_methods = set(self.rpc_table.keys()) | \
             {'initialize', 'initialized', 'shutdown', 'exit'}  # see self.verify_initialization()
@@ -1068,13 +1127,16 @@ class Server:
 
 
     def register_service_rpc(self, name, method):
-        """Registers a service request """
+        """Registers a service request, i.e. a call that will be accepted from
+        a second connection. Otherwise, requests coming from a new connection
+        if a connection has already been established, will be rejected, because
+        language servers only accept one client at a time."""
         name = name[:name.find('(')]
         if name in self.rpc_table:
             self.log('Service {} is shadowed by an rpc-call with the same name.'.format(name))
         else:
             self.rpc_table[name] = method
-            # self.known_methods.add(name)
+            self.service_calls.add(name)
 
     def start_logging(self, filename: str = "") -> str:
         """Starts logging to a file. If `filename` is void or a directory
@@ -1114,7 +1176,7 @@ class Server:
         if self.log_file:
             append_log(self.log_file, *args, echo=self.echo_log)
 
-    def rpc_identify_server(self, service_call: bool = False, *args, **kwargs):
+    def rpc_identify_server(self, service_call: bool = False, html: bool = False, *args, **kwargs):
         """Returns an identification string for the server."""
         identify = "DHParser %s %s under %s" \
                    % (__version__, self.server_name, identify_python())
@@ -1122,8 +1184,8 @@ class Server:
 
     def rpc_logging(self, *args, **kwargs) -> str:
         """Starts logging with either a) the default filename, if args is
-         empty or the empty string b) the given log file name if `args[0]`
-         is a non-empty string c) stops logging if `args[0]` is `None`.
+         empty or the empty string; or b) the given log file name if `args[0]`
+         is a non-empty string; or c) stops logging if `args[0]` is `None`.
          """
         if len(args) > 0:
             log_name = args[0]
@@ -1143,8 +1205,54 @@ class Server:
         else:
             return self.start_logging()
 
-    async def run(self, method_name: str, method: Callable, params: Union[Dict, Sequence]) \
-            -> Tuple[Optional[JSON_Type], Optional[RPC_Error_Type]]:
+    def rpc_info(self, service_call: bool = False, html: bool = False,
+                 *args, **kwargs) -> str:
+        """Returns information on the implemented LSP- and service-functions."""
+        info = rpc_table_info(self.rpc_table, html)
+        if html:
+            return HTML_TEMPLATE.format(heading=f'{self.server_name} API:', content=info)
+        return info
+
+    def rpc_serve_page(self, file_path: str,
+                       service_call: bool = False, html: bool = False,
+                       *args, **kwargs) -> Union[str, bytes]:
+        """Loads and returns the HTML page or other files stored in file `file_path`"""
+        def error(err_desc: str, details: str = '') -> str:
+            return HTML_TEMPLATE.format(heading=err_desc, content=details)
+
+        i = file_path.find(':')
+        if i >= 0:
+            chksum = file_path[:i]
+            file_path = file_path[i + 1:]
+        else:
+            chksum = ''
+        dirname, basename = os.path.split(file_path)
+        allow_file = os.path.join(dirname, 'DHParser.allow')
+        if not os.path.isfile(allow_file):
+            return error(f'Access to "{file_path}" forbidden!',
+                         '"DHParser.allow not found.')
+        if not os.path.isfile(file_path):
+            return error(f'Page "{file_path}" not found on.', '')
+        with open(allow_file, 'r', encoding='utf-8') as f:
+            lines = [l.split('=') for l in f.read().split('\n') if l.strip()]
+        allow = { l[0].strip(): l[1].strip() for l in lines if len(l) == 2}
+        if not chksum or allow[basename] != chksum:
+            try:
+                with open(file_path, 'r', encoding='utf-8') as f:  page = f.read()
+                if is_html_name(basename):
+                    chksum = md5(page)
+                    allow[basename] = chksum
+                    with open(allow_file, 'w', encoding='utf-8') as f:
+                        f.writelines(''.join([name, '=', chk, '\n'])
+                                     for name, chk in allow.items())
+                    page = re.sub('<body.*?>', f'<body chksum="{chksum}">', page)
+            except UnicodeDecodeError:
+                with open(file_path, 'rb') as f:  page = f.read()
+            return page
+        return ''  # empty string means that page has not changed
+
+    async def run(self, method_name: str, method: Callable, params: Union[Dict, List, Tuple]) \
+            -> Tuple[Optional[Union[JSON_Type, BytesType]], Optional[RPC_Error_Type]]:
         """Picks the right execution method (process, thread or direct execution) and
         runs it in the respective executor. In case of a broken ProcessPoolExecutor it
         restarts the ProcessPoolExecutor and tries to execute the method again.
@@ -1159,7 +1267,7 @@ class Server:
         result, rpc_error = await self.exec.execute(executor, method, params)
         return result, rpc_error
 
-    async def respond(self, writer: StreamWriterType, response: Union[str, bytes]):
+    async def respond(self, writer: StreamWriterType, response: Union[str, BytesType]):
         """Sends a response to the given writer. Depending on the configuration,
         the response will be logged. If the response appears to be a json-rpc
         response a JSONRPC_HEADER will be added depending on
@@ -1185,16 +1293,16 @@ class Server:
                 self.connection.alive = False
 
     def amend_service_call(self, func_name: str, func: Callable, argument: Union[Tuple, Dict],
-                           err_func: Callable) -> Tuple[Callable, Union[Tuple, Dict]]:
+                           err_func: Callable, html: bool = False) -> Tuple[Callable, Union[Tuple, Dict]]:
         if argument is None:
             argument = ()
         if getattr(func, '__self__', None) == self:
             if isinstance(argument, Dict):
                 params = argument.copy()
-                params.update({'service_call': True})
+                params.update({'service_call': True, 'html': html})
                 return func, params
             else:
-                return func, argument + (True,)
+                return func, argument + (True, html)
         else:
             return err_func, {} if isinstance(argument, Dict) else ()
 
@@ -1209,9 +1317,10 @@ class Server:
         except ValueError:
             header = b''
 
-        async def respond(response: str):
+        async def respond(response: Union[str, bytes, bytearray]):
             if header:
-                await self.respond(writer, JSONRPC_HEADER % len(response) + response)
+                HEADER = JSONRPC_HEADER if isinstance(response, str) else JSONRPC_HEADER_BYTES
+                await self.respond(writer, HEADER % len(response) + response)
             else:
                 await self.respond(writer, response)
 
@@ -1239,11 +1348,17 @@ class Server:
             err_func = lambda *args, **kwargs: \
                 'No function named "%s" known to server %s !' % (func_name, self.server_name)
             func = self.rpc_table.get(func_name, err_func)  # type: Callable
+            rpc_error = None
             if service_call:
-                func, argument = self.amend_service_call(func_name, func, argument, err_func)
-            result, rpc_error = await self.run(func_name, func, argument)
+                if func_name in self.service_calls:
+                    func, argument = self.amend_service_call(func_name, func, argument, err_func)
+                else:
+                    rpc_error = ('', f'Functions "{func_name}" has been requested from a "'
+                                     f'second connection although it is not a service call!')
             if rpc_error is None:
-                if isinstance(result, str):
+                result, rpc_error = await self.run(func_name, func, argument)
+            if rpc_error is None:
+                if isinstance(result, (str, bytes, bytearray)):
                     await respond(result)
                 elif result is not None:
                     try:
@@ -1266,33 +1381,44 @@ class Server:
         else:
             m = re.match(RE_GREP_URL, data)
             if m:
-                func_name, argument = m.group(1).decode().strip('/').split('/', 1) + [None]
+                # TODO: use urllib to parse parameters
+                func_name, argument = (m.group(1).decode().strip('/').split('/', 1) + [None])[:2]
+                # really bad hack to determine mime-type of response
+                mime = 'text/html'
+                if argument:
+                    if argument[-4:].lower() == '.css':  mime = 'text/css'
+                    elif argument[-3:].lower() == '.js':  mime = 'text/javascript'
+                argument = (argument,) if argument else ()
                 if func_name.encode() == STOP_SERVER_REQUEST_BYTES:
                     await self.respond(
-                        writer, http_response(ONELINER_HTML.format(line=self.stop_response)))
+                        writer, http_response(HTML_TEMPLATE.format(heading=self.stop_response,
+                                                                   content='')))
                     self.kill_switch = True
                     reader.feed_eof()
                 else:
-                    err_func = lambda _: UNKNOWN_FUNC_HTML.format(func=func_name)
+                    err_func = lambda *args, **kwargs: UNKNOWN_FUNC_HTML.format(func=func_name)
                     func = self.rpc_table.get(func_name, err_func)
                     if service_call:
+                        print('ServiceCall: ' + func_name + str(argument))
                         func, argument = self.amend_service_call(
-                            func_name, func, argument, err_func)
-                    result, rpc_error = await self.run(func.__name__, func,
-                                                       (argument,) if argument else ())
+                            func_name, func, argument, err_func, html=True)
+                    elif func_name in self.service_calls:
+                        print('Pseudo ServiceCall: ' + func_name + str(argument))
+                        argument += (False, True)  # service_call = False, html = True
+                    result, rpc_error = await self.run(func.__name__, func, argument)
                     if rpc_error is None:
                         if result is None:
                             result = ''
-                        if isinstance(result, str):
-                            await self.respond(writer, http_response(result))
+                        if isinstance(result, (str, bytes, bytearray)):
+                            await self.respond(writer, http_response(result, mime))
                         else:
                             try:
                                 await self.respond(writer, http_response(
-                                    pp_json(result, cls=DHParser_JSONEncoder)))
+                                    pp_json(result, cls=DHParser_JSONEncoder), 'text/plain'))
                             except TypeError as err:
-                                await self.respond(writer, http_response(str(err)))
+                                await self.respond(writer, http_response(str(err), 'text/plain'))
                     else:
-                        await self.respond(writer, http_response(rpc_error[1]))
+                        await self.respond(writer, http_response(rpc_error[1], 'text/plain'))
         assert self.connection
         self.connection.task_done(task_id)
 
@@ -1351,8 +1477,13 @@ class Server:
                 pass  # no errors in result
             if json_id is not None and result is not None:
                 try:
-                    json_result = {"jsonrpc": "2.0", "result": result, "id": json_id}
-                    await self.respond(writer, json_dumps(json_result, cls=DHParser_JSONEncoder))
+                    if isinstance(result, (bytes, bytearray)):
+                        # assume that result is a json-format byte-string!
+                        response = b'{"jsonrpc": "2.0", "result": %s, "id": %i}' % (result, json_id)
+                    else:
+                        json_result = {"jsonrpc": "2.0", "result": result, "id": json_id}
+                        response = json_dumps(json_result, cls=DHParser_JSONEncoder)
+                    await self.respond(writer, response)
                 except TypeError as err:
                     rpc_error = -32070, str(err)
 
@@ -1395,8 +1526,8 @@ class Server:
 
         def connection_alive() -> bool:
             """-> `False` if connection is dead or shall be shut down."""
-            return not self.kill_switch and self.connection and self.connection.alive \
-                   and not reader.at_eof()  # and not reader.closed
+            return not self.kill_switch and self.connection is not None \
+                   and self.connection.alive and not reader.at_eof()  # and not reader.closed
 
         buffer = bytearray()  # type: bytearray
         while connection_alive():
@@ -1780,13 +1911,13 @@ def _run_tcp_server(host, port, rpc_functions: RPC_Type,
     server.run_tcp_server(host, port)
 
 
-ConcurrentType = TypeVar('Concurrent', Thread, Process)
+ConcurrentType = TypeVar('ConcurrentType', Thread, Process)
 
 
 def spawn_tcp_server(host: str = USE_DEFAULT_HOST,
                      port: int = USE_DEFAULT_PORT,
                      parameters: Union[Tuple, Dict, Callable] = echo_requests,
-                     Concurrent: Generic[ConcurrentType] = Process) -> ConcurrentType:
+                     Concurrent: ConcurrentType = Process) -> ConcurrentType:
     """
     Starts DHParser-Server that communicates via tcp in a separate process
     or thread. Can be used for writing test code.
