@@ -34,10 +34,15 @@ import operator
 from typing import AbstractSet, ByteString, Callable, cast, Container, Dict, \
     Tuple, List, Sequence, Union, Text
 
+try:
+    import cython
+except ImportError:
+    import DHParser.externallibs.shadow_cython as cython
+
 from DHParser.error import ErrorCode, AST_TRANSFORM_CRASH, ERROR
 from DHParser.nodetree import Node, WHITESPACE_PTYPE, TOKEN_PTYPE, LEAF_PTYPES, PLACEHOLDER, \
     RootNode, parse_sxpr, flatten_sxpr, TreeContext
-from DHParser.toolkit import issubtype, isgenerictype, expand_table, smart_list, re, cython, \
+from DHParser.toolkit import issubtype, isgenerictype, expand_table, smart_list, re, \
     deprecation_warning
 
 
